@@ -146,6 +146,7 @@ class ConfigureProblem:
                 ImplicitConstraintFcn.TAU_EQUALS_INVERSE_DYNAMICS,
                 node=Node.ALL_SHOOTING,
                 constraint_type=ConstraintType.IMPLICIT,
+                phase=nlp.phase_idx,
             )
         if implicit_soft_contacts:
             ConfigureProblem.configure_soft_contact_forces(nlp, False, True)
@@ -171,6 +172,7 @@ class ConfigureProblem:
                 ImplicitConstraintFcn.SOFT_CONTACTS_EQUALS_SOFT_CONTACTS_DYNAMICS,
                 node=Node.ALL_SHOOTING,
                 constraint_type=ConstraintType.IMPLICIT,
+                phase=nlp.phase_idx,
             )
 
     @staticmethod
@@ -218,6 +220,7 @@ class ConfigureProblem:
                 ImplicitConstraintFcn.TAU_EQUALS_INVERSE_DYNAMICS,
                 node=Node.ALL_SHOOTING,
                 constraint_type=ConstraintType.IMPLICIT,
+                phase=nlp.phase_idx,
             )
         if implicit_soft_contacts:
             ConfigureProblem.configure_soft_contact_forces(nlp, False, True)
@@ -242,6 +245,7 @@ class ConfigureProblem:
                 ImplicitConstraintFcn.SOFT_CONTACTS_EQUALS_SOFT_CONTACTS_DYNAMICS,
                 node=Node.ALL_SHOOTING,
                 constraint_type=ConstraintType.IMPLICIT,
+                phase=nlp.phase_idx,
             )
 
     @staticmethod
@@ -336,7 +340,7 @@ class ConfigureProblem:
         ConfigureProblem.configure_soft_contact_function(ocp, nlp)
 
     @staticmethod
-    def configure_dynamics_function(ocp, nlp, dyn_func, **extra_params):
+    def configure_dynamics_function(ocp, nlp, dyn_func, expand: bool = True, **extra_params):
         """
         Configure the dynamics of the system
 
@@ -348,6 +352,8 @@ class ConfigureProblem:
             A reference to the phase
         dyn_func: Callable[states, controls, param]
             The function to get the derivative of the states
+        expand: bool
+            If the dynamics should be expanded with casadi
         """
 
         nlp.parameters = ocp.v.parameters_in_list
@@ -361,7 +367,9 @@ class ConfigureProblem:
             [dynamics],
             ["x", "u", "p"],
             ["xdot"],
-        ).expand()
+        )
+        if expand:
+            nlp.dynamics_func = nlp.dynamics_func.expand()
 
     @staticmethod
     def configure_contact_function(ocp, nlp, dyn_func: Callable, **extra_params):
