@@ -1,6 +1,7 @@
 from itertools import chain
+from math import inf
 from typing import Union, Callable
-from time import time
+from time import perf_counter
 
 import numpy as np
 import biorbd_casadi as biorbd
@@ -12,7 +13,7 @@ from ..limits.constraints import ConstraintFcn
 from ..limits.objective_functions import ObjectiveFcn
 from ..limits.path_conditions import InitialGuess, Bounds
 from ..misc.enums import SolverType, InterpolationType
-from ..interfaces.SolverOptions import Solver
+from ..interfaces.solver_options import Solver
 
 
 class RecedingHorizonOptimization(OptimalControlProgram):
@@ -62,8 +63,11 @@ class RecedingHorizonOptimization(OptimalControlProgram):
         warm_start: Solution = None,
         solver_first_iter: Solver.Generic = None,
         export_options: dict = None,
+        max_consecutive_failing: int = inf,
+        update_function_extra_params: dict = None,
+        get_all_iterations: bool = False,
         **advance_options,
-    ) -> Solution:
+    ) -> Union[Solution, tuple]:
         """
         Solve MHE program. The program runs until 'update_function' returns False. This function can be used to
         modify the objective set, for instance. The warm_start_function can be provided by the user. Otherwise, the
@@ -88,6 +92,12 @@ class RecedingHorizonOptimization(OptimalControlProgram):
             A Solution to initiate the first iteration from
         export_options: dict
             Any options related to the saving of the data at each iteration
+        max_consecutive_failing: int
+            The number of consecutive failing before stopping the nmpc. Default is infinite
+        update_function_extra_params: dict
+            Any parameters to pass to the update function
+        get_all_iterations: bool
+            If an extra output value that includes all the individual solution should be returned
         advance_options: Any
             The extra options to pass to the advancing methods
 
@@ -113,14 +123,21 @@ class RecedingHorizonOptimization(OptimalControlProgram):
         self._initialize_frame_to_export(export_options)
 
         total_time = 0
-        real_time = 0
+        real_time = perf_counter()
+        all_solutions = []
+        consecutive_failing = 0
+        update_function_extra_params = {} if update_function_extra_params is None else update_function_extra_params
 
         self.total_optimization_run = 0
-        while update_function(self, self.total_optimization_run, sol):
+        while (
+            update_function(self, self.total_optimization_run, sol, **update_function_extra_params)
+            and consecutive_failing < max_consecutive_failing
+        ):
             sol = super(RecedingHorizonOptimization, self).solve(
                 solver=solver_current,
                 warm_start=warm_start,
             )
+            consecutive_failing = 0 if sol.status == 0 else consecutive_failing + 1
 
             # Set the option for the next iteration
             if self.total_optimization_run == 0:
@@ -137,26 +154,28 @@ class RecedingHorizonOptimization(OptimalControlProgram):
             warm_start = None
 
             total_time += sol.real_time_to_optimize
-            if self.total_optimization_run == 0:
-                real_time = time()  # Skip the compile time (so skip the first call to solve)
+            if solver_current == Solver.ACADOS and self.total_optimization_run == 0:
+                real_time = perf_counter()  # Reset timer to skip the compiling time (so skip the first call to solve)
 
             # Solve and save the current window
             _states, _controls = self.export_data(sol)
             states.append(_states)
             controls.append(_controls)
+            if get_all_iterations:
+                all_solutions.append(sol)
 
             # Update the initial frame bounds and initial guess
             self.advance_window(sol, **advance_options)
 
             self.total_optimization_run += 1
 
-        real_time = time() - real_time
+        real_time = perf_counter() - real_time
 
         # Prepare the modified ocp that fits the solution dimension
         sol = self._initialize_solution(states, controls)
         sol.solver_time_to_optimize = total_time
         sol.real_time_to_optimize = real_time
-        return sol
+        return (sol, all_solutions) if get_all_iterations else sol
 
     def _initialize_frame_to_export(self, export_options):
         if export_options is None:
@@ -316,7 +335,7 @@ class CyclicRecedingHorizonOptimization(RecedingHorizonOptimization):
         cyclic_options: dict = None,
         solver_first_iter: Solver.Generic = None,
         **extra_options,
-    ) -> Solution:
+    ) -> Union[Solution, tuple]:
 
         if solver is None:
             solver = Solver.ACADOS()
